#/usr/bin/pdksh
#
#  server_push.sh
#
#    This is the main driver.  It pushes the file listed in $PAYLOAD_FILE to
#    the remote server and executes the command in $PAYLOAD on each
#    server/userid in the file optionally specified in the SERVER_LIST
#    environment variable.  If SERVERLIST is not set, it defaults
#    to "server_list.txt" in the current directory.  The SERVER_LIST file
#    should contain lines in the format "servername  [userid]" where userid
#    is optional and will default to "oracle" if not specified.
#
#    Adjust TARGET_DIR, PAYLOAD_FILE, and PAYLOAD prior to running this script.
#
#set -x
#
echo "ENV settings coming in:"
echo "TAG="$TAG
echo "TARGET_DIR="$TARGET_DIR
echo "PAYLOAD_FILE="$PAYLOAD_FILE
echo "PAYLOAD_CMD="$PAYLOAD_CMD
echo "CLEANUP_CMD="$CLEANUP_CMD
#
echo `date '+%Y-%m-%d %H:%M'` - getting SERVER_LIST
export P1=$1
if [[ $P1"x" == "x" ]] ; then
  echo "SERVER_LIST not passed on command-line."
else
  export SERVER_LIST=$P1
fi
if [[ $SERVER_LIST"x" == "x" ]] ; then
  echo "SERVER_LIST was not set in .profile....using default"
  export SERVER_LIST=../server_list.txt
fi
echo "SERVER_LIST="$SERVER_LIST
#
export P2=$2
if [[ $P2"x" == "x" ]] ; then
  echo "LOG_DATE not passed on command-line."
else
  export LOG_DATE=$P2
fi
if [[ $LOG_DATE"x" == "x" ]] ; then
  echo "LOG_DATE was not set in the environment...using TODAY."
  LOG_DATE=`date +%Y-%m-%d`
fi
echo "LOG_DATE="$LOG_DATE
#
#define output directory
#
echo `date '+%Y-%m-%d %H:%M'` - setting up output directory
LOGDIR='logs.'${LOG_DATE}
export LOGDIR
echo "LOGDIR="${LOGDIR}
if [ ! -d $LOGDIR ] ; then
  mkdir $LOGDIR
  echo "LOGDIR created"
fi
#
#loop thru list of remote servers
#
echo `date '+%Y-%m-%d %H:%M'` - server_push.sh - start
# the following sed command should be sed 's/^M//'
cat ${SERVER_LIST}|egrep -v "^#"|egrep -v "^$" | sed 's/^M//'| \
  awk --compat '{if (NF==1) {print $1 " unknownIP oracle 2"} else {if (NF==2) {print $1 " " $2 " oracle 2"} else {if (NF==3) {print $1 " " $2 " " $3 " 2"} else {print $1 " " $2 " " $3 " " $4}}}}' | \
while read hid ip uid sshver
do
  push_failed='N'
  use_ip='N'
  LOGFILE=${LOGDIR}/${hid}_${uid}_${TAG}
  if [ ! `echo ${PAYLOAD_FILE}|awk '{print $1 "X"}'` = "noneX" ] ; then
    echo `date '+%Y-%m-%d %H:%M'` ${uid}\@${hid} - pushing payload_files to server
    scp -o protocol=${sshver} ${PAYLOAD_FILE} ${uid}@${hid}:${TARGET_DIR}
    ret=$?
    if [ ${ret} = 1 ] ; then
      echo `date '+%Y-%m-%d %H:%M'` ${uid}\@${hid} - ERROR RC=${ret} on payload_file push -- trying ${ip}
      use_ip='Y'
      scp -o protocol=${sshver} ${PAYLOAD_FILE} ${uid}@${ip}:${TARGET_DIR}
      ret=$?
    fi
    if [ ! ${ret} = 0 ] ; then
      echo `date '+%Y-%m-%d %H:%M'` ${uid}\@${hid} - ERROR RC=${ret} on payload_file push
      push_failed='Y'
    fi
  fi
  if [ ${push_failed} = 'N' ] ; then
    if [ ${use_ip} = 'N' ] ; then
      echo `date '+%Y-%m-%d %H:%M'` ${uid}\@${hid} - executing payload
      ssh -n -o protocol=${sshver}  ${uid}@${hid} ${PAYLOAD_CMD} > ${LOGDIR}/${hid}_${uid}_${TAG}
    else
      echo `date '+%Y-%m-%d %H:%M'` ${uid}\@${hid} - executing payload via ${ip}
      ssh -n -o protocol=${sshver}  ${uid}@${ip}  ${PAYLOAD_CMD} > ${LOGDIR}/${hid}_${uid}_${TAG}
    fi
    ret=$?
    if [ ${ret} = 255 ] ; then
      if [ ${use_ip} = 'N' ] ; then
        echo `date '+%Y-%m-%d %H:%M'` ${uid}\@${hid} - ERROR RC=${ret} on payload_file execute -- trying ${ip}
        use_ip='Y'
        ssh -n -o protocol=${sshver}  ${uid}@${ip} ${PAYLOAD_CMD} > ${LOGDIR}/${hid}_${uid}_${TAG}
        ret=$?
      fi
    fi
    if [ ! ${ret} = 0 ] ; then
      echo `date '+%Y-%m-%d %H:%M'` ${uid}\@${hid} - ERROR RC=${ret} on payload execute
    fi
    if [ ! `echo ${CLEANUP_CMD}|awk '{print $1 "X"}'` = "noneX" ] ; then
      if [ ${use_ip} = 'N' ] ; then
        echo `date '+%Y-%m-%d %H:%M'` ${uid}\@${hid} - executing cleanup
        ssh -n -o protocol=${sshver}  ${uid}@${hid} ${CLEANUP_CMD}
      else
        echo `date '+%Y-%m-%d %H:%M'` ${uid}\@${hid} - executing cleanup via ${ip}
        ssh -n -o protocol=${sshver}  ${uid}@${ip}  ${CLEANUP_CMD}
      fi
    fi
  fi
done
echo "   "
echo `date '+%Y-%m-%d %H:%M'` - server_push.sh - end
